@@ -2,7 +2,10 @@
 from emora_stdm.state_transition_dialogue_manager.dialogue_flow \
     import DialogueFlow, module_source_target, module_state
 from emora_stdm.state_transition_dialogue_manager.macros_common import *
-from emora_stdm.state_transition_dialogue_manager.knowledge_base import KnowledgeBase
+from emora_stdm.state_transition_dialogue_manager.utilities import \
+    json_serialize_flexible, json_deserialize_flexible
+from emora_stdm.state_transition_dialogue_manager.dialogue_flow import \
+    speaker_enum_mapping, speaker_enum_rmapping
 from time import time
 import dill
 from pathos.multiprocessing import ProcessingPool as Pool
@@ -57,7 +60,7 @@ class CompositeDialogueFlow:
                 assert next_state is not None
                 self.controller().set_state(next_state)
             except Exception as e:
-                print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self.controller(), self.controller().state()))
+                print('Error in CompositeDialogueFlow. Component: {}  State: {}'.format(self.controller_name(), self.controller().state()))
                 traceback.print_exc(file=sys.stdout)
                 response, next_state = '', self._system_error_state
                 visited = visited - {next_state}
@@ -232,3 +235,25 @@ class CompositeDialogueFlow:
 
     def state(self):
         return self._controller_name, self._controller.state()
+
+    def serialize(self):
+        """
+        Returns json serialized dict of
+            {'vars': vars, 'gates': gates, 'state': state}
+        """
+        config = {'vars': self._controller.vars(),
+                 'gates': self._controller.gates(),
+                 'state': self.state()}
+        return json_serialize_flexible(config, speaker_enum_mapping)
+
+    def deserialize(self, config_str):
+        config = json_deserialize_flexible(config_str, speaker_enum_rmapping)
+        self.reset()
+        self.set_state(config['state'])
+        self.set_vars(config['vars'])
+        for name,component in self._components.items():
+            component.set_gates(config['gates'])
+
+    def new_turn(self, toplevel="SYSTEM"):
+        self.reset()
+        self.set_controller(toplevel)
