@@ -14,12 +14,12 @@ import traceback
 import spacy
 import sys
 try:
-    nlp = spacy.load("en_core_web_sm")
+    nlp = spacy.load("en_core_web_md")
 except Exception as e:
     traceback.print_exc()
     print('Error loading Spacy', file=sys.stderr)
     print('Please run the following command:', file=sys.stderr)
-    print('python -m spacy download en_core_web_sm', file=sys.stderr)
+    print('python -m spacy download en_core_web_md', file=sys.stderr)
 try:
     nltk.data.find('wordnet')
 except:
@@ -28,9 +28,9 @@ try:
     nltk.find('averaged_perceptron_tagger')
 except:
     nltk.download('averaged_perceptron_tagger')
-from nltk.corpus import wordnet
 from emora_stdm.state_transition_dialogue_manager.wordnet import \
     related_synsets, wordnet_knowledge_base, lemmas_of
+from nltk.corpus import wordnet
 import regex
 
 
@@ -263,7 +263,7 @@ class CheckVarsConjunction(Macro):
     def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
         for arg in args:
             var, val = _assignment_to_var_val(arg)
-            if (var not in vars and val != 'None') or (vars[var] != val):
+            if (var not in vars and val != 'None') or (var in vars and vars[var] != val):
                 return False
         return True
 
@@ -342,35 +342,43 @@ class Equal(Macro):
                 return False
         return True
 
+
 class Gate(Macro):
     def __init__(self, dialogue_flow):
         self.dialogue_flow = dialogue_flow
     def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        configuration = ConfigurationDict()
-        requirements = {}
+        configuration = {}
         for arg in args:
-            if isinstance(arg, str) and ':' in arg:
+            if ':' in arg:
                 var, val = arg.split(':')
-                var = var.strip()
-                val = val.strip()
-                if val == 'None':
-                    val = None
-                requirements[var] = val
+            elif '=' in arg:
+                var, val = arg.split('=')
+                if var[0] == '$':
+                    var = var[1:]
+            else:
+                var, val = arg, None
+            if val is None:
                 if var in vars:
                     configuration[var] = vars[var]
                 else:
-                    configuration[var] = None
+                    return False
+            elif val == 'None':
+                if var in vars and vars[var] is not None and vars[var] != 'None':
+                    return False
             else:
-                if arg in vars:
-                    configuration[arg] = vars[arg]
-                else:
-                    configuration[arg] = None
-        self.dialogue_flow.set_gate_requirements(requirements)
-        if self.dialogue_flow.passes_gate(configuration):
-            self.dialogue_flow.buffer_configuration(configuration)
-            return True
-        else:
-            return False
+                if var not in vars or vars[var] != val:
+                    return False
+        vars['__gate__'] = configuration
+        return True
+
+
+class Unset(Macro):
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        for var in args:
+            if var in vars and vars[var] is not None and vars[var] != 'None':
+                return False
+        return True
 
 
 class Clear(Macro):
@@ -440,116 +448,239 @@ class TokLimit(Macro):
         else:
             return False
 
-class Agree(Macro):
-    """
-
-    """
-    def __init__(self):
-        self.natex = NatexNLU(agree)
-        self.natex.compile()
-        self.natex._regex = self.natex.regex().replace("_END_","").strip()
-
-    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        return self.natex.regex()
-
-class Disagree(Macro):
-    """
-
-    """
-    def __init__(self):
-        self.natex = NatexNLU(disagree)
-        self.natex.compile()
-        self.natex._regex = self.natex.regex().replace("_END_","").strip()
-
-    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        return self.natex.regex()
-
-class Question(Macro):
-    """
-
-    """
-    def __init__(self):
-        self.natex = NatexNLU(question)
-        self.natex.compile()
-        self.natex._regex = self.natex.regex().replace("_END_","").strip()
-
-    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        return self.natex.regex()
-
-class Negation(Macro):
-    """
-
-    """
-    def __init__(self):
-        self.natex = NatexNLU(negation)
-        self.natex.compile()
-        self.natex._regex = self.natex.regex().replace("_END_","").strip()
-
-    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        return self.natex.regex()
-
-class DontKnow(Macro):
-    """
-
-    """
-
-    def __init__(self):
-        dont_know = '[{' \
-                    'dont know,do not know,unsure,[not,{sure,certain}],hard to say,no idea,uncertain,[!no {opinion,opinions,idea,ideas,thought,thoughts,knowledge}],' \
-                    '[{dont,do not}, have, {opinion,opinions,idea,ideas,thought,thoughts,knowledge}],' \
-                    '[!{cant,cannot,dont} {think,remember,recall}]' \
-                    '}]'
-        self.natex = NatexNLU(dont_know)
-        self.natex.compile()
-        self.natex._regex = self.natex.regex().replace("_END_", "").strip()
-
-
-    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        return self.natex.regex()
-
-
-class Maybe(Macro):
-    """
-
-    """
-
-    def __init__(self):
-        maybe = '[{maybe,possibly,sort of,kind of,kinda,a little,at times,sometimes,could be,potentially,its possible}]'
-        self.natex = NatexNLU(maybe)
-        self.natex.compile()
-        self.natex._regex = self.natex.regex().replace("_END_", "").strip()
-
-    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        return self.natex.regex()
-
 class Transition(Macro):
     """
 
     """
+    def __init__(self, dialogue_flow):
+        self.dialogue_flow = dialogue_flow
+
     def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
         if len(args) > 1:
-            vars['__transition_score__'] = float(args[1])
+            score = float(args[1])
         else:
-            vars['__transition_score__'] = 0.5
-        vars['__transition__'] = args[0]
+            score = 0.5
+        target = args[0]
         vars['__converged__'] = 'True'
+        self.dialogue_flow.dynamic_transitions().append(
+            (NatexNLU('/.*/'),
+             (self.dialogue_flow.state(), target, self.dialogue_flow.speaker()),
+             score))
 
-class Unexpected(Macro):
+class VirtualTransitions(Macro):
 
-    def __init__(self):
-        self.question_natex = NatexNLU(question)
+    def __init__(self, dialogue_flow):
+        self.dialogue_flow = dialogue_flow
 
     def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
-        if self.question_natex.match(ngrams.text()):
-            if '_explained_stupidity_' in vars and vars['_explained_stupidity_'] == 'True':
-                vars['__response_prefix__'] = 'I\'m not really sure at the moment.'
+        for state in args:
+            if ':' in state:
+                controller, state = state.split(':')
+                extraction_df = self.dialogue_flow.composite_dialogue_flow().component(controller)
+                for source, target, speaker in extraction_df.transitions(state):
+                    natex = extraction_df.transition_natex(source, target, speaker)
+                    score = extraction_df.transition_settings(source, target, speaker).score
+                    if not isinstance(source, tuple):
+                        source = (controller, source)
+                    if not isinstance(target, tuple):
+                        target = (controller, target)
+                    self.dialogue_flow.dynamic_transitions().append(
+                        (natex, (source, target, speaker), score))
             else:
-                vars['_explained_stupidity_'] = 'True'
-                vars['__response_prefix__'] = 'Sorry, I don\'t think I understand your question. ' \
-                                              'There\'s still a lot I\'m trying to figure out. '
+                for source, target, speaker in self.dialogue_flow.transitions(state):
+                    natex = self.dialogue_flow.transition_natex(source, target, speaker)
+                    score = self.dialogue_flow.transition_settings(source, target, speaker).score
+                    self.dialogue_flow.dynamic_transitions().append(
+                        (natex, (source, target, speaker), score))
+        return False
+
+
+class Intent(Macro):
+
+    def _similarity(self, user_utterance, dev_utterance):
+        return user_utterance.similarity(dev_utterance)
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        for i, arg in enumerate(args):
+            if isinstance(arg, str) and arg.isnumeric():
+                threshold = float(arg)
+                del args[i]
+                break
         else:
-            vars['__response_prefix__'] = 'Yeah.'
-        return True
+            threshold = 0.0
+        user = nlp(ngrams.text())
+        dev = [nlp(arg) for arg in args]
+        similarity = max([self._similarity(user, x) for x in dev])
+        vars['__score__'] = similarity
+        if similarity < threshold:
+            return False
+        else:
+            return True
+
+
+class Default(Macro):
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        vars['__score__'] = 0.001
+        return ''
+
+
+class GoalPursuit(Macro):
+    """
+    Begin pursuing a goal.
+
+    This sets the current goal to the specified goal, and any outstanding current goal
+    will be saved by being pushed onto the goal stack.
+
+    Goals pushed onto the stack are of the form:
+    [goal id str, return state str, return phrase str, doom counter int]
+    """
+
+    def __init__(self, goal_exit_macro):
+        self.goal_exit_macro = goal_exit_macro
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        if '__stack__' not in vars:
+            vars['__stack__'] = []
+        if '__goal__' not in vars:
+            vars['__goal__'] = 'None'
+        if '__goal_return_state__' not in vars:
+            vars['__goal_return_state__'] = 'None'
+        goal = args[0]
+        self.goal_exit_macro.run(ngrams, vars, args)
+        vars['__goal__'] = goal
+
+
+class GoalCompletion(Macro):
+    """
+    Complete either the current goal, or a specified goal that
+    is either the current goal or the first matching on the stack.
+    """
+
+    def __init__(self, dialogue_flow):
+        self.dialogue_flow = dialogue_flow
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        goal = None
+        if len(args) == 1:
+            goal = args[0]
+        if goal is None or goal == vars['__goal__']:
+            vars['__goal__'] = 'None'
+        else:
+            stack = vars['__stack__']
+            for i in range(len(stack) - 1, -1, -1):
+                if stack[i][0] == goal:
+                    del stack[i]
+                    break
+
+class GoalExit(Macro):
+    """
+    Exit a goal, pushing the current goal onto the stack.
+    """
+
+    def __init__(self, dialogue_flow):
+        self.dialogue_flow = dialogue_flow
+        self.default_return_phrase = ''
+        self.default_doom_counter = None
+
+    def goal_return_state(self, goal, vars):
+        if '__goal_return_state__' in vars and vars['__goal_return_state__'] != 'None':
+            grs = vars['__goal_return_state__']
+            vars['__goal_return_state__'] = 'None'
+            return grs
+        elif goal in self.dialogue_flow.goals() and self.dialogue_flow.goals()[goal]['return_state']:
+            return self.dialogue_flow.goals()[goal]['return_state']
+        else:
+            return vars['__system_state__']
+
+    def goal_return_phrase(self, goal, vars):
+        if '__goal_return_phrase__' in vars and vars['__goal_return_phrase__'] != 'None':
+            grp = vars['__goal_return_phrase__']
+            vars['__goal_return_phrase__'] = 'None'
+            return grp
+        elif goal in self.dialogue_flow.goals() and self.dialogue_flow.goals()[goal]['return_phrase']:
+            return self.dialogue_flow.goals()[goal]['return_phrase']
+        else:
+            return ''
+
+    def goal_doom_counter(self, goal, vars):
+        if '__goal_doom_counter__' in vars and vars['__goal_doom_counter__'] != 'None':
+            gdc = vars['__goal_doom_counter__']
+            vars['__goal_doom_counter__'] = 'None'
+            return gdc
+        elif goal in self.dialogue_flow.goals() and self.dialogue_flow.goals()[goal]['doom_counter']:
+            return self.dialogue_flow.goals()[goal]['doom_counter']
+        else:
+            return 'None'
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        if '__goal__' not in vars:
+            vars['__goal__'] = 'None'
+        if vars['__goal__'] != 'None':
+            push_goal_id = vars['__goal__']
+            push_goal_state = self.goal_return_state(push_goal_id, vars)
+            push_goal_return_phrase = self.goal_return_phrase(push_goal_id, vars)
+            push_goal_doom_counter = self.goal_doom_counter(push_goal_id, vars)
+            push_goal = [push_goal_id, push_goal_state, push_goal_return_phrase, push_goal_doom_counter]
+            vars['__stack__'].append(push_goal)
+        vars['__goal__'] = 'None'
+
+
+class GoalReturn(Macro):
+    """
+    Return to the specified goal, or the first goal on the stack.
+
+    Specifying a goal is used for situations where control is taken from a
+    goal flow, but the sub-convo actually ends up serving one of the goals
+    in the stack. In this case, a transition to the appropriate goal flow
+    point can be taken with GoalResume(resumed_goal) called in order to
+    return to the goal without any awkward return phrase.
+
+    An optional second argument can be used to return to a specific state.
+    An optional third argument can be used for a desired return phrase.
+    """
+
+    def __init__(self, dialogue_flow):
+        self.dialogue_flow = dialogue_flow
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        goal = None
+        if len(args) > 0 and args[0] != 'None':
+            goal = args[0]
+        vars['__goal__'] = 'None'
+        while vars['__stack__']:
+            id, state, phrase, doom = vars['__stack__'].pop()
+            if doom != 'None' and doom <= 0:
+                continue
+            if goal is not None and goal != id:
+                continue
+            if len(args) > 1 and args[1] != 'None' and args[1] != '':
+                state = args[1]
+            if len(args) > 2:
+                phrase = args[2]
+            vars['__target__'] = state
+            vars['__goal__'] = id
+            return phrase
+
+
+class SetGoalReturnPoint(Macro):
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        vars['__goal_return_state__'] = args[0]
+
+
+class Target(Macro):
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        vars['__target__'] = args[0]
+
+
+
+
+
+
+
+
 
 
 
